/*
  Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

  File:      cqo1.cs

  Purpose:   Demonstrates how to solve a small conic qaudratic
  optimization problem using the MOSEK API.
*/

using System;

 
public class cqo1
{
  public static void Main ()
    {
      const int NUMCON = 1;
      const int NUMVAR = 6;
      const int NUMANZ = 4;
      // Since the value infinity is never used, we define
      // 'infinity' symbolic purposes only
      double infinity = 0;
      
      mosek.boundkey[] bkc    = { mosek.boundkey.fx };
      double[] blc = { 1.0 };
      double[] buc = { 1.0 };
      
      mosek.boundkey[] bkx = {mosek.boundkey.lo,
                   mosek.boundkey.lo,
                   mosek.boundkey.lo,
                   mosek.boundkey.lo,
                   mosek.boundkey.fr,
                   mosek.boundkey.fr};
      double[] blx = { 0.0,
                       0.0,
                       0.0,
                       -infinity,
                       -infinity};
      double[] bux = { 0.0,
                       0.0,
                       0.0,
                       +infinity,
                       +infinity};
      
      double[] c   = { 0.0,
                       0.0,
                       0.0,
                       0.0,
                       1.0,
                       1.0};
        
      double[] aval   = {1.0,
                         1.0,
                         1.0,
                         1.0};
      int[]    asub   = {0,
                         0,
                         0,
                         0};
      int[] ptrb = {0,1,2,3,5,5};
      int[] ptre = {1,2,3,4,5,5};
      
      int[] csub = new int[3];
   
      double[] xx  = new double[NUMVAR];

      mosek.Env
        env = null;
      mosek.Task
        task = null;
   
      try
        {
          // create a new environment object
          env  = new mosek.Env ();
          // if you like, do some pre-initialization here
          // (e.g. setting up exit or stream callback)
          // then initialize the environment for use
          env.init ();
          // create a task object
          task = new mosek.Task (env, NUMCON, NUMVAR);

                  
          task.inputdata(NUMCON,NUMVAR,
                         c,0.0,
                         ptrb,
                         ptre,
                         asub,
                         aval,
                         bkc,
                         blc,
                         buc,
                         bkx,
                         blx,
                         bux);
                
          csub[0] = 4;
          csub[1] = 0;
          csub[2] = 2;
          task.appendcone(mosek.conetype.quad,
                          0.0, /* For future use only, can be set to 0.0 */
                          csub);

          csub[0] = 5;
          csub[1] = 1;
          csub[2] = 3;
          task.appendcone(mosek.conetype.quad,0.0,csub);

          Console.Write ("putintparam");
          task.putobjsense(mosek.objsense.minimize);

          Console.Write ("optimize");
          try
            {
              task.optimize();
            }
                     
          catch (mosek.Exception e)
            {
              Console.WriteLine (e);
            }

          Console.Write ("solutionsummary");
          task.solutionsummary(mosek.streamtype.msg);

          Console.Write ("getsolutionslice");
          task.getsolutionslice(mosek.soltype.itr, /* Interior solution.       */
                                mosek.solitem.xx,  /* Which part of solution.  */
                                0,                 /* Index of first variable. */
                                NUMVAR,            /* Index of last variable+1 */
                                xx);
          for(int j = 0; j < NUMVAR; ++j)
            Console.WriteLine ("x[{0}]:{1}", j,xx[j]);

        }
      catch (mosek.Exception e)
        {
          Console.WriteLine (e);
        }

      if (task != null) task.Dispose ();
      if (env  != null)  env.Dispose ();
    }
}
